@@ -39,7 +39,7 @@ static inline struct ifa * kif_get_primary_ip(struct iface *i UNUSED) { return N
 #define EA_KRT_SCOPE		EA_CODE(PROTOCOL_KERNEL, 0x12)
 
 
-#define KRT_METRICS_MAX		0x10	/* RTAX_QUICKACK+1 */
+#define KRT_METRICS_MAX		0x12	/* RTAX_FASTOPEN_NO_COOKIE+1 */
 #define KRT_METRICS_OFFSET	0x20	/* Offset of EA_KRT_* vs RTAX_* */
 
 #define KRT_FEATURES_MAX	4
@@ -54,7 +54,7 @@ static inline struct ifa * kif_get_primary_ip(struct iface *i UNUSED) { return N
 #define EA_KRT_WINDOW		EA_CODE(PROTOCOL_KERNEL, 0x23)
 #define EA_KRT_RTT		EA_CODE(PROTOCOL_KERNEL, 0x24)
 #define EA_KRT_RTTVAR		EA_CODE(PROTOCOL_KERNEL, 0x25)
-#define EA_KRT_SSTRESH		EA_CODE(PROTOCOL_KERNEL, 0x26)
+#define EA_KRT_SSTHRESH		EA_CODE(PROTOCOL_KERNEL, 0x26)
 #define EA_KRT_CWND		EA_CODE(PROTOCOL_KERNEL, 0x27)
 #define EA_KRT_ADVMSS		EA_CODE(PROTOCOL_KERNEL, 0x28)
 #define EA_KRT_REORDERING 	EA_CODE(PROTOCOL_KERNEL, 0x29)
@@ -64,6 +64,8 @@ static inline struct ifa * kif_get_primary_ip(struct iface *i UNUSED) { return N
 #define EA_KRT_RTO_MIN		EA_CODE(PROTOCOL_KERNEL, 0x2d)
 #define EA_KRT_INITRWND		EA_CODE(PROTOCOL_KERNEL, 0x2e)
 #define EA_KRT_QUICKACK		EA_CODE(PROTOCOL_KERNEL, 0x2f)
+#define EA_KRT_CONGCTL		EA_CODE(PROTOCOL_KERNEL, 0x30)
+#define EA_KRT_FASTOPEN_NO_COOKIE EA_CODE(PROTOCOL_KERNEL, 0x31)
 
 
 struct krt_params {
